# Refill device constants
from typing import List, Dict

import settings

DRUG_CALIBRATION_PENDING = "Pending"
DRUG_CALIBRATION_DONE = "Done"
WEIGHING_LIMITATION_OF_REFILL_DEVICE = 525000  # weight in mg

# Canister constants
BIG_CANISTER_CODE = 70
SMALL_CANISTER_CODE = 71

CANISTER_LENGTH = 69
CANISTER_BREADTH = 69
BIG_CANISTER_HEIGHT = 156
SMALL_CANISTER_HEIGHT = 56
CANISTER_USABLE_VOLUME_PERCENT = 90

GROUP_MASTER_MFD_CANISTER = 14
GROUP_MASTER_MFD_CANISTER_DRUG = 15

MFD_DRUG_PENDING_STATUS = 60
MFD_DRUG_FILLED_STATUS = 61
MFD_DRUG_SKIPPED_STATUS = 62
MFD_DRUG_DROPPED_STATUS = 63
MFD_DRUG_RTS_REQUIRED_STATUS = 110
MFD_DRUG_RTS_DONE = 111
MFD_DRUG_MVS_FILLED = 152

MFD_STATIONS = 4
MFD_CANISTER_PENDING_STATUS = 64
MFD_CANISTER_IN_PROGRESS_STATUS = 65
MFD_CANISTER_FILLED_STATUS = 66
MFD_CANISTER_VERIFIED_STATUS = 67
MFD_CANISTER_SKIPPED_STATUS = 68
MFD_CANISTER_DROPPED_STATUS = 69
MFD_CANISTER_RTS_DONE_STATUS = 112
MFD_CANISTER_RTS_REQUIRED_STATUS = 113
MFD_CANISTER_MVS_FILLING_REQUIRED = 150
MFD_CANISTER_MVS_FILLED_STATUS = 151
REASON_PACK_PRIORITY_CHANGES = "Pack Priority Changed"

MFD_ACTION_FILLED = 26
MFD_ACTION_DROPPED = 27
MFD_ACTION_SKIP = 28
MFD_ACTION_PILLS_EXTRACTED = 29
MFD_ACTION_FILLING_IN_PROGRESS = 30
MFD_ACTION_DEACTIVATE_AND_SKIP = 31

MFD_ACTION_STATUS_MAP = {
    MFD_CANISTER_PENDING_STATUS: MFD_ACTION_PILLS_EXTRACTED,
    MFD_CANISTER_IN_PROGRESS_STATUS: MFD_ACTION_FILLING_IN_PROGRESS,
    MFD_CANISTER_FILLED_STATUS: MFD_ACTION_FILLED,
    MFD_CANISTER_VERIFIED_STATUS: MFD_ACTION_FILLED,
    MFD_CANISTER_SKIPPED_STATUS: MFD_ACTION_SKIP,
    MFD_CANISTER_DROPPED_STATUS: MFD_ACTION_DROPPED,
    MFD_CANISTER_RTS_DONE_STATUS: MFD_ACTION_PILLS_EXTRACTED,
    MFD_CANISTER_RTS_REQUIRED_STATUS: MFD_ACTION_SKIP,
    MFD_CANISTER_MVS_FILLING_REQUIRED: MFD_ACTION_SKIP,
    MFD_CANISTER_MVS_FILLED_STATUS: MFD_ACTION_DROPPED,
}

MFD_CANISTER_ACTIVATED = 36
MFD_CANISTER_DEACTIVATED = 37
MFD_CANISTER_ADDED = 38
MFD_CANISTER_DELETED = 39
MFD_CANISTER_MARKED_MISPLACED = 40
MFD_CANISTER_FOUND = 41

MFD_QUAD_CANISTER_CAPACITY = 20
GROUP_MASTER_BATCH_MFD = 16

MFD_CANISTER_DONE_LIST = [MFD_CANISTER_FILLED_STATUS, MFD_CANISTER_VERIFIED_STATUS]

MFD_CANISTER_DONE_LIST_PQ = [MFD_CANISTER_FILLED_STATUS, MFD_CANISTER_VERIFIED_STATUS, MFD_CANISTER_RTS_REQUIRED_STATUS]

PACK_MFD_REQUIRED_LIST = [MFD_CANISTER_FILLED_STATUS, MFD_CANISTER_VERIFIED_STATUS, MFD_CANISTER_PENDING_STATUS,
                          MFD_CANISTER_IN_PROGRESS_STATUS, MFD_CANISTER_DROPPED_STATUS,
                          MFD_CANISTER_RTS_REQUIRED_STATUS]

MFD_BATCH_PENDING = 95
MFD_BATCH_IN_PROGRESS = 96
MFD_BATCH_FILLED = 104
MFD_BATCH_PRE_SKIPPED = 269

MFS_STATIONS = []
MFS_GROUP_1_LINKED = ['63', '64']
MFS_GROUP_2_LINKED = []

CONST_MANUAL_FILL_STATION_DOC_ID = 'MFS_V1'
CONST_MANUAL_FILL_STATION_TYPE = 'MFS'

CONST_MFD_RTS_DOC_ID = 'MFD_RTS_V1'
CONST_MFD_RTS_TYPE = 'MFD_RTS'

CANISTER_LABEL_PREFIX = 'CAN'
MFD_CANISTER_LABEL_PREFIX = 'M'
MFD_TROLLEY_CANISTER_CAPACITY = 160

SERIAL_NUMBER_SEPARATOR = '#'
SUCCESS_RESPONSE = "success"
TZ_UTC = "+00:00"

# RTS codes
RTS_GROUP_ID = 21  # RtsStatus
RTS_VERIFIED = 82  # Verified
RTS_REJECTED = 83  # Rejected
RTS_NOT_SURE = 84  # Not Sure
RTS_SKIPPED_MAPPED = 85  # Skipped And Mapped
RTS_USER_SKIPPED = 108  # User Skipped
RTS_SYSTEM_SKIPPED = 109  # Rts Skipped

# Reuse status
REUSE_REQUIRED = 282
REUSE_IN_PROGRESS = 283
REUSE_DONE = 284
RTS_DONE = 285
RESEALED = 286
DRUG_DISCARDED = 287

REUSE_DRUG_STATUS = 64  # GROUP_MASTER CODE

# PVS User Station codes
US_STATION_GROUP_ID = 8  # PVS
US_STATION_REJECTED = 39  # Slot rejected
US_STATION_SUCCESS = 40  # Slot success
US_STATION_NOT_SURE = 41  # Not Sure
US_STATION_DELETED = 42  # Slot deleted
US_STATION_MISSING_PILLS = 43  # Missing Pill
US_STATION_MISSING_PILLS_AND_DELETED = 44  # Missing Pills and Deleted

# PVS Classification Status codes
PVS_CLASSIFICATION_STATUS_GROUP_ID = 22  # PvsClassificationStatus
PVS_CLASSIFICATION_STATUS_NOT_MATCHED = 86  # Not Matched
PVS_CLASSIFICATION_STATUS_MATCHED = 87  # Matched
PVS_CLASSIFICATION_STATUS_NOT_SURE = 88  # Not Sure
PVS_CLASSIFICATION_STATUS_EXTRA_PILL = 89  # Extra Pill
PVS_CLASSIFICATION_STATUS_PILL_DIMENSION_NOT_REGISTERED = 105  # dimension not registered of the pill
PVS_CLASSIFICATION_STATUS_PILL_DIMENSION_CONFUSING = 106  # pill dimension confusing
PVS_CLASSIFICATION_STATUS_PILL_CONFUSION_EXCLAMATION = 107
PVS_CLASSIFICATION_STATUS_BROKEN_PILL = 230
PVS_CLASSIFICATION_STATUS_MAY_BE_BROKEN_PILL = 231


SLOT_COLOUR_STATUS_GROUP_ID = 50
SLOT_COLOUR_STATUS_RED = 213 #0
SLOT_COLOUR_STATUS_GREEN = 214 #1
SLOT_COLOUR_STATUS_WHITE = 232
SLOT_COLOUR_STATUS_BLUE = 235
SLOT_COLOUR_STATUS_RED_BLUE = 236

PACK_VERIFICATION_STATUS_GROUP_ID = 49
PACK_NOT_MONITORED_STATUS = 210 #0
PACK_MONITORED_STATUS = 211 #1
PACK_VERIFIED_STATUS = 212 #2



# PRS Classification Status Codes
PRS_DRUG_STATUS_GROUP_ID = 23
PRS_DRUG_STATUS_PENDING = 90
PRS_DRUG_STATUS_DONE = 91
PRS_DRUG_STATUS_REJECTED = 92
PRS_DRUG_STATUS_VERIFIED = 93
PRS_DRUG_STATUS_REGISTERED = 94

PRS_IMAGE_COUNT = 4

# PRS modes
RTS_MODE = 'rts'
NON_RTS_MODE = 'non_rts'
MINIMUM_IMAGE_COUNT_PRS_COMPANY_SETTING = "MINIMUM_IMAGE_COUNT_FOR_PRS"

CSR_SERIAL_NUMBER = 13
MFS_SERIAL_NUMBER = 81

# Types of locations in Drawer
LOCATION_TYPE = {"ACTIVE": "active", "DISABLED": "disabled", "EMPTY": "empty"}

# canister and container constants
SIZE_OR_TYPE_BIG = 70
SIZE_OR_TYPE_SMALL = 71
USAGE_FAST_MOVING = 72
USAGE_MEDIUM_FAST_MOVING = 73
USAGE_MEDIUM_SLOW_MOVING = 74
USAGE_SLOW_MOVING = 75
SIZE_OR_TYPE_MFD = 76

# canister and container constants dict
TYPE_OR_SIZE_DICT = {"Big": 76, "Small": 77}
USAGE_DICT = {"Fast Moving": 72, "Medium Fast Moving": 73, "Medium Slow Moving": 74, "Slow Moving": 75}

# Container Serial number series dict based on station type: KEY- station_type, value-container serial number series
CONTAINER_SERIAL_NUMBER_SERIES_DICT = {13: "CRD"}

# Device Serial number series dict based on station type: KEY- station_type, value-device serial number series
DEVICE_SERIAL_NUMBER_SERIES_DICT = {13: "CSR", 11: "MFD"}

STATION_ID_MAX_LENGTH = 5  # EXAMPLE: CSR00001 so this 00001 represents station id

PRIMARY_PCB_ID = 1
SECONDARY_PCB_ID = 2

LOCATIONS_PER_PCB_IN_CSR = 10

INVENTORY_STATUS_CODE_OK = 200
INVENTORY_STATUS_CODE_BAD_REQUEST = 400
INVENTORY_STATUS_CODE_DATA_NOT_FOUND = 404
INVENTORY_STATUS_CODE_CONNECTION_ERROR = 4003
NULL_RFID_CSR = '00000000000000000000000000000000'

LOGGED_IN_USER_INFO_COUCH_DB_DOC = 'logged_in_user_info'
CSR_DRAWER_EMLOCK_COUCH_DB_DOC = 'csr-drawer-emlock-status'

CSR_LIFT_TROLLEY_LEVELS = [8, 9, 10]

MFD_TROLLEY_MIDDLE_LEVEL = 4
RTS_PACK_MANUAL_REASON = 'Marked as Manual'
RTS_PACK_DELETE_REASON = 'Deleted'

USER_MFD_CANISTER_STATUS = {
    MFD_CANISTER_RTS_REQUIRED_STATUS: 'RTS Required',
    MFD_CANISTER_PENDING_STATUS: 'Empty',
    MFD_CANISTER_FILLED_STATUS: 'Filled',
    MFD_CANISTER_IN_PROGRESS_STATUS: 'In Progress',
    MFD_CANISTER_MVS_FILLING_REQUIRED: 'Manual Filling Required',
}

# ASRS
ACTION_PACK_REMOVED_FROM_CONTAINER = 22
ASRS_DEVICE_LINKED_CAPACITY = 3

# Guided Canister Transfer flow
LIFT_TROLLEY_DRAWERS_COUNT = 12
LIFT_TROLLEY_LOCATIONS_PER_DRAWER = 10
NORMAL_TROLLEY_DRAWERS_COUNT = 16
NORMAL_TROLLEY_LOCATIONS_PER_DRAWER = 20

# empty drawers to choose from after canister testing in CSR
POST_TESTING_DRAWER_RECOMMENDATION = ['C-1', 'C-2', 'D-1', 'D-2', 'E-1', 'E-2', 'F-1', 'F-2', 'G-1', 'G-2', 'H-1',
                                      'H-2', 'I-1', 'I-2', 'J-1', 'J-1']

# GuidedMeta
GUIDED_META_GROUP_ID = 31  # Guided Meta
GUIDED_META_RECOMMENDATION_DONE = 114  # Guided Tx Recommendation Done
GUIDED_META_DRUG_BOTTLE_FETCHED = 115  # Guided Tx Drug Bottle Fetched
GUIDED_META_TO_TROLLEY_DONE = 116  # Guided Tx To Trolley Done
GUIDED_META_REPLENISH_DONE = 117  # Guided Tx Replenish Done
GUIDED_META_TO_ROBOT_DONE = 118  # Guided Tx To Robot Done
GUIDED_META_TO_CSR_DONE = 119  # Guided Tx To CSR Done

# GuidedTrackerFlow
GUIDED_TRACKER_GROUP_ID = 32
GUIDED_TRACKER_PENDING = 120
GUIDED_TRACKER_REPLENISH = 121
GUIDED_TRACKER_REPLACED = 122
GUIDED_TRACKER_SKIPPED = 123  # skip replenish for canister
GUIDED_TRACKER_DONE = 124
GUIDED_TRACKER_SKIPPED_AND_REPLACED = 125
GUIDED_TRACKER_SKIPPED_AND_DONE = 126
GUIDED_TRACKER_TRANSFER_SKIPPED = 140
GUIDED_TRACKER_TRANSFER_AND_REPLENISH_SKIPPED = 143

GUIDED_TRACKER_TO_TROLLEY_SKIPPED = 179
GUIDED_TRACKER_TO_TROLLEY_SKIPPED_AND_ALTERNATE = 180
GUIDED_TRACKER_TO_TROLLEY_ALTERNATE_TRANSFER_LATER = 181
GUIDED_TRACKER_TO_ROBOT_SKIPPED = 182
GUIDED_TRACKER_TO_ROBOT_SKIPPED_AND_ALTERNATE = 183
GUIDED_TRACKER_TO_ROBOT_ALTERNATE_TRANSFER_LATER = 184
GUIDED_TRACKER_TO_CSR_SKIPPED = 185
GUIDED_TRACKER_DRUG_SKIPPED = 186
GUIDED_TRACKER_TO_TROLLEY_ALTERNATE = 187
GUIDED_TRACKER_TO_ROBOT_ALTERNATE = 188
GUIDED_TRACKER_TO_TROLLEY_TRANSFER_LATER_SKIP = 189
GUIDED_TRACKER_TO_ROBOT_TRANSFER_LATER_SKIP = 190
GUIDED_TRACKER_TO_TROLLEY_DONE = 191

GUIDED_TRACKER_TRANSFER_ACTION = 33
GUIDED_TRACKER_SKIPPED_ACTION = 34
GUIDED_TRACKER_SKIPPED_AND_DEACTIVATE_ACTION = 35

guided_tracker_existing_status_list = [GUIDED_TRACKER_TO_TROLLEY_SKIPPED_AND_ALTERNATE,
                                       GUIDED_TRACKER_TO_ROBOT_SKIPPED_AND_ALTERNATE,
                                       GUIDED_TRACKER_TO_TROLLEY_TRANSFER_LATER_SKIP,
                                       GUIDED_TRACKER_TO_ROBOT_TRANSFER_LATER_SKIP,
                                       GUIDED_TRACKER_TO_CSR_SKIPPED]

guided_tracker_skipped_canister_code_list = [GUIDED_TRACKER_SKIPPED, GUIDED_TRACKER_SKIPPED_AND_DONE,
                                             GUIDED_TRACKER_SKIPPED_AND_REPLACED]

guided_tracker_get_existing_data = [GUIDED_TRACKER_TO_ROBOT_SKIPPED_AND_ALTERNATE,
                                    GUIDED_TRACKER_TO_TROLLEY_SKIPPED_AND_ALTERNATE]

guided_tracker_to_trolley_status_list = [GUIDED_TRACKER_PENDING, GUIDED_TRACKER_TO_TROLLEY_ALTERNATE]
guided_tracker_to_robot_status_list = [GUIDED_TRACKER_SKIPPED, GUIDED_TRACKER_PENDING, GUIDED_TRACKER_REPLENISH,
                                       GUIDED_TRACKER_TO_TROLLEY_ALTERNATE, GUIDED_TRACKER_TO_ROBOT_ALTERNATE,
                                       GUIDED_TRACKER_TO_TROLLEY_DONE, GUIDED_TRACKER_SKIPPED_AND_DONE]
guided_tracker_to_csr_status_list = [GUIDED_TRACKER_REPLACED, GUIDED_TRACKER_SKIPPED_AND_REPLACED,
                                     GUIDED_TRACKER_TRANSFER_SKIPPED, GUIDED_TRACKER_TO_ROBOT_SKIPPED,
                                     GUIDED_TRACKER_TO_ROBOT_SKIPPED_AND_ALTERNATE,
                                     GUIDED_TRACKER_TO_ROBOT_ALTERNATE_TRANSFER_LATER,
                                     GUIDED_TRACKER_TO_ROBOT_ALTERNATE, GUIDED_TRACKER_TO_ROBOT_TRANSFER_LATER_SKIP]
guided_tracker_to_robot_skipped_codes = [GUIDED_TRACKER_TRANSFER_SKIPPED, GUIDED_TRACKER_TO_ROBOT_SKIPPED,
                                         GUIDED_TRACKER_TO_ROBOT_SKIPPED_AND_ALTERNATE]

GUIDED_TRANSFER_DOCUMENT_NAME = 'guided_canister_transfer'

# Pre- precessing db doc
PRE_PROCESSING_WIZARD = 'pre-processing-wizard-sequence'
REPLENISH_WIZARD = "replenish-wizard"
MFD_TRANSFERS_ERROR = "mfd-transfers-error"

# Guided transfer couch db docs
GUIDED_TRANSFER_WIZARD = 'guided-transfer-wizard'
# canister transfer couch db doc name
CANISTER_TRANSFER_WIZARD = 'canister-transfer-wizard'

GUIDED_FETCH_DRUGS_MODULE = 0
GUIDED_TO_TROLLEY_MODULE = 1
GUIDED_REPLENISH_MODULE = 2
GUIDED_TO_DRUG_DISPENSER_MODULE = 3
GUIDED_TO_CSR_MODULE = 4

ROBOT_USER = 1

MFD_TROLLEY_EMPTY_DRAWER_LEVEL = [1, 2, 3, 4]
MFD_TROLLEY_FILLED_DRAWER_LEVEL = [5, 6, 7, 8]

MFD_TROLLEY_EMPTY_DRAWER_TYPE = 'Empty'
MFD_TROLLEY_RTS_DRAWER_TYPE = 'RTS'

MFD_DRAWER_TYPE_ID = 76

MFD_ERROR_MESSAGES = {
    'NO_CAN_REQ': 'No canister required.',
    'DRAWER_SCAN_REQ': 'Canister is missing. Kindly place the canister having ID: {} and follow the scan and'
                       ' transfer flow.',
    'WRONG_CAN_DETECTED': 'Wrong canister detected. Kindly place the canister having ID: {} to continue filling.',
    'MISSING_CAN': 'Canister is missing. Kindly place the canister having ID: {} to continue filling.',
    'WRONG_DRAWER_SCANNED': 'Wrong canister removed. Kindly place the canister having ID: {} and follow the scan and '
                            'transfer flow.',
    'TROLLEY_PLACEMENT_REQUIRED': 'Please transfer the canister having ID: {} to the Drawer: {} of {}.',
    'WRONG_PLACEMENT': 'Please place the canister having ID: {} on {} location.',
    'WRONG_HOME_CART': 'Wrong canister detected. Kindly place the canister of {}.',
    'DEACTIVATE_CANISTER': 'Deactivated canister detected. Kindly activate it or replace it with another canister to '
                           'proceed further.',
}

# string constants
STRING_COMPANY = "company"
STRING_SYSTEM = "system"

# session constants
DRUG_DISPENSING_TEMPLATES_MODULE = 26  # DrugDispensingTemplatesModule
BATCH_SCHEDULING_MODULE = 27  # BatchSchedulingModule
PACK_PRE_PROCESSING_MODULE = 28  # PackPreProcessingModule

MODULE_TYPE_DDT_SCREEN = 1
MODULE_TYPE_SELECT_FACILITY_SCREEN = 2
MODULE_TYPE_SELECT_SYSTEMS_SCREEN = 3
MODULE_TYPE_ALTERNATE_DRUG_SCREEN = 4
MODULE_TYPE_OVERLOAD_PACK_SCREEN = 5
MODULE_TYPE_RECOMMEND_DISTRIBUTION_SCREEN = 6
MODULE_TYPE_SCHEDULED_BATCH_SCREEN = 7
MODULE_TYPE_RECOMMEND_ALTERNATE_DRUG_SCREEN = 8
MODULE_TYPE_CANISTER_RECOMMENDATION_SCREEN = 9
MODULE_TYPE_IMPORT_BATCH_SCREEN = 10
MODULE_TYPE_RPH_SCREEN = 41

MODULE_META_SYSTEM_SELECTION = 1
MODULE_META_ALTERNATE_DRUG_SELECTION_AND_DESELECTION = 2
MODULE_META_USER_SELECTION_AND_SYSTEM_SELECTION = 3
MODULE_META_ALTERNATE_DRUG_MODIFICATION = 4
MODULE_META_CANISTER_TRANSFERS = 5
MODULE_META_USER_SELECTION_FULLY_MANUAL_PACKS = 6
MODULE_META_CANISTER_TRANSFER_MISPLACED_CANISTER = 7

CONST_MFD_PRE_FILL_DOC_ID = 'mfd-canister-pre-fill'
CONST_MFD_PRE_FILL_DOC_TYPE = 'mfd-canister-pre-fill-v2'

MFD_MODULES = {
    'SCHEDULED_BATCH': 0,
    'DRUG_LIST': 1
}

AUTH_MFS_SYSTEM_TYPE = "MFS"
AUTH_MF_SYSTEM_TYPE = "MF"

# dummy robot constants
TRANSFER_TYPE_DEVICE_TO_SHELF = "device_to_shelf"
TRANSFER_TYPE_SHELF_TO_DEVICE = "shelf_to_device"
TRANSFER_TYPE_DEVICE_TO_DEVICE = "normal"
TRANSFER_TYPE_SWAP = "swap"

# ---------------------------------------------------------------- #
# Canister Transfer
KEY_TRANSFER_TO_TROLLEY = "transfer_to_trolley"
KEY_TRANSFER_FROM_TROLLEY = "transfer_from_trolley"
KEY_TROLLEY_SCANNED = "trolley_scanned"
KEY_DRAWER_SCANNED = "drawer_scanned"
KEY_DRAWER_DATA = "drawer_data"
KEY_TOTAL_CANISTERS_TO_BE_TRANSFERRED = "total_canisters_to_be_transferred"
CANISTER_TRANSFER_WIZARD_DOCUMENT_NAME = "canister-transfer-wizard"
CANISTER_TRANSFER_DEVICE_DOCUMENT_NAME = "canister_transfer"

CANISTER_TRANSFER_TO_TROLLEY_MODULE = 0
CANISTER_TRANSFER_TO_ROBOT_MODULE = 1
CANISTER_TRANSFER_TO_CSR_MODULE = 2

CANISTER_TRANSFER_GROUP_ID = 33

CANISTER_TRANSFER_RECOMMENDATION_DONE = 127
CANISTER_TRANSFER_TO_TROLLEY_DONE = 128
CANISTER_TRANSFER_TO_ROBOT_DONE = 129
CANISTER_TRANSFER_TO_CSR_DONE = 163

CANISTER_TX_STATUS_GROUP_ID = 40

CANISTER_TX_PENDING = 164

CANISTER_TX_TO_TROLLEY_DONE = 165
CANISTER_TX_TO_TROLLEY_TRANSFERRED_MANUALLY = 166
CANISTER_TX_TO_TROLLEY_SKIPPED = 167
CANISTER_TX_TO_TROLLEY_SKIPPED_AND_ALTERNATE = 168
CANISTER_TX_TO_TROLLEY_ALTERNATE = 169
CANISTER_TX_TO_TROLLEY_ALTERNATE_TRANSFER_LATER = 170

CANISTER_TX_TO_ROBOT_DONE = 171
CANISTER_TX_TO_ROBOT_SKIPPED = 172
CANISTER_TX_TO_ROBOT_SKIPPED_AND_ALTERNATE = 173
CANISTER_TX_TO_ROBOT_ALTERNATE = 174
CANISTER_TX_TO_ROBOT_ALTERNATE_TRANSFER_LATER = 175

CANISTER_TX_TO_CSR_DONE = 176
CANISTER_TX_TO_CSR_SKIPPED = 177
CANISTER_TX_TO_ROBOT_ALTERNATE_TRANSFER_AT_PPP = 178

# canister transfer actions
CANISTER_TX_TRANSFER_ACTION = 23
CANISTER_TX_SKIPPED_ACTION = 24
CANISTER_TX_SKIPPED_AND_DEACTIVATE_ACTION = 25

canister_tx_existing_status_list = [CANISTER_TX_TO_TROLLEY_SKIPPED_AND_ALTERNATE,
                                    CANISTER_TX_TO_ROBOT_SKIPPED_AND_ALTERNATE]

PER_QUAD_CANISTER_BUFFER = 5
# ------------------------------------------------------------- #

MFD_CANISTER_TRANSFER_WIZARD_DOCUMENT_NAME = "mfd-transfer-wizard"

# wizard values for mfd transfers
# MFD_TRANSFER_WIZARD_MODULES = {
#     "RTS_SCREEN_WIZARD": 0,
#     "MISPLACED_CANISTER_WIZARD": 1,
#     "EMPTY_CANISTER_WIZARD": 2,
#     "TRANSFER_TO_ROBOT_WIZARD": 3
# }

MFD_TRANSFER_WIZARD_MODULES = {
    "RTS_SCREEN_WIZARD": 0,
    "EMPTY_CANISTER_WIZARD": 1,
    "TRANSFER_TO_ROBOT_WIZARD": 2
}

MFD_TRANSFER_COUCH_DB = "mfd_transfer"
MFD_TRANSFER_DOC_TYPE = "mfd_canister_transfer"

PACK_GRID_V3_DICT_ACCORDING_TO_V2 = {(0, 0): "1", (1, 0): "2", (2, 0): "3", (3, 0): "4", (4, 0): "5", (5, 0): "6",
                                     (6, 0): "7", (6, 1): "8", (5, 1): "9", (4, 1): "10", (3, 1): "11", (2, 1): "12",
                                     (1, 1): "13", (0, 1): "14", (0, 2): "15", (1, 2): "16", (2, 2): "17", (3, 2): "18",
                                     (4, 2): "19", (5, 2): "20", (6, 2): "21", (6, 3): "22", (5, 3): "23", (4, 3): "24",
                                     (3, 3): "25", (2, 3): "26", (1, 3): "27", (0, 3): "28", }

# printer_type_id constants
PRINTER_TYPE_CRM = 141
PRINTER_TYPE_USER_STATION = 142
DEFAULT_PRINTER_TYPE = PRINTER_TYPE_USER_STATION

LINK_STATION_MESSAGE = "Please link the adjacent stations or deselect any one of them before selecting the new station."
STATION_MAX_LIMIT_MESSAGE = "Station selection reaches to its max limit."

# Canister Quantity update type
CANISTER_QUANTITY_UPDATE_TYPE_GROUP_ID = 37
CANISTER_QUANTITY_UPDATE_TYPE_REPLENISHMENT = 148
CANISTER_QUANTITY_UPDATE_TYPE_ADJUSTMENT = 149
CANISTER_QUANTITY_UPDATE_TYPE_DISCARD = 253

# replenish mode
REPLENISH_MODE_GROUP_ID = 20
REFILL_DEVICE = 80
MANUAL_CODE = 81
REPLENISHED_WITH_ENTIRE_DRUG_BOTTLE = 147
ADJUST_REPLENISH_MODE_PRIORITY = [MANUAL_CODE, REPLENISHED_WITH_ENTIRE_DRUG_BOTTLE, REFILL_DEVICE]

# to add 3 scan type
SCAN_TYPE_GROUP_ID = 19
USER_ENTERED = 77
BARCODE = 78
DATA_MATRIX = 79
CANISTER_SCAN = 256
QR_SCAN = 273
UPC_SCAN = 275
REUSE_PACK_SCAN = 298
VIAL_SCAN = 299
CASE_ID_PREFIX = 'DPARS'
ITEM_ID_PREFIX = 'DPRD'

# to add pack type
PACK_TYPE = 58
PACK_GRID_TYPE = 62
WEEKLY_UNITDOSE_PACK = 263
WEEKLY_MULTIDOSE_PACK = 264

# to add pack fill type
PACK_FILL_TYPE = 61
MULTI_DOSE_PER_PACK = 267
UNIT_DOSE_PER_PACK = 268

# canister testing constants
CANISTER_TESTING_GROUP_ID = 36
CANISTER_TESTING_PASS = 145
CANISTER_TESTING_FAIL = 146
CANISTER_TESTING_PENDING = 198

CANISTER_FAIL_DUE_TO_NON_COMPATIBLE_DRUG = "drug not compatible"
CANISTER_FAIL_DUE_TO_BROKEN = "Broken canister"

DEACTIVE_REASON_ISSUE_WITH_EERPROM = "Issue with Canister EEPROM"
DEACTIVE_REASON_BROKEN_CANISTER = "Broken Canister"

# canister status history constants
GROUP_MASTER_CANISTER_STATUS = 39

CODE_MASTER_CANISTER_ACTIVATE = 156
CODE_MASTER_CANISTER_DEACTIVATE = 157
CODE_MASTER_CANISTER_REACTIVATE = 158
CODE_MASTER_CANISTER_NDC_CHANGE = 159
CODE_MASTER_CANISTER_NEW_FROM_OLD = 160


# ips prescription
REQUIRED_PATIENT_INFO_KEYS = ["file_name", "pharmacy_facility_id", "pharmacy_patient_id", "facility_name",
                              "patient_name", "patient_picture", "address1", "city", "state", "zip_code",
                              "workphone", "dob", "patient_allergy", "patient_no", "pharmacy_fill_id",
                              "filled_by", "fill_start_date", "fill_end_date", "rx_details", "ips_username",
                              "daycare_program"]

REQUIRED_PATIENT_INFO_VALUES = ["file_name", "pharmacy_facility_id", "pharmacy_patient_id", "facility_name",
                                "patient_name", "address1", "zip_code",
                                "dob", "patient_no", "pharmacy_fill_id",
                                "fill_start_date", "fill_end_date", "rx_details", "ips_username"]

REQUIRED_RX_KEYS = ["pharmacy_doctor_id", "doctor_name", "doctor_phone", "pharmacy_drug_id", "pharmacy_rx_no",
                    "caution1", "caution2", "remaining_refill", "drug_name", "ndc", "daw_code", "delivery_schedule",
                    "delivery_date", "delivery_time", "sig", "is_tapered", "total_qty", "fill_details", "morning",
                    "noon", "evening", "bed"]

REQUIRED_RX_VALUES = ["pharmacy_doctor_id", "doctor_name", "pharmacy_rx_no", "remaining_refill", "ndc", "daw_code",
                      "delivery_schedule",
                      "sig", "is_tapered", "fill_details", "morning",
                      "noon", "evening", "bed"]

REQUIRED_FILL_DETAILS_KEYS = ["quantity", "hoa_time"]

REQUIRED_FILL_DETAILS_VALUES = ["quantity", "hoa_time"]

MFD_CANISTER_STATUS_ACTION = {
    "ACTIVATE": "ACTIVATE",
    "ADD": "ADD",
    "DEACTIVATE": "DEACTIVATE",
    "DELETE": "DELETE",
}

MFD_CANISTER_INACTIVE = 0
MFD_CANISTER_ACTIVE = 1
MFD_CANISTER_MISPLACED = 2

MFD_STATE_STATUS = {
    MFD_CANISTER_INACTIVE: 'Inactive',
    MFD_CANISTER_ACTIVE: 'Active',
    MFD_CANISTER_MISPLACED: 'Misplaced',
}

MFD_STATE_FILTERS_LIST = {
    MFD_CANISTER_INACTIVE: [MFD_CANISTER_INACTIVE],
    MFD_CANISTER_ACTIVE: [MFD_CANISTER_ACTIVE, MFD_CANISTER_MISPLACED],
    MFD_CANISTER_MISPLACED: [MFD_CANISTER_MISPLACED],
}

MFD_STATE_LIST = {
    MFD_CANISTER_INACTIVE: [MFD_CANISTER_INACTIVE],
    MFD_CANISTER_ACTIVE: [MFD_CANISTER_ACTIVE],
    MFD_CANISTER_MISPLACED: [MFD_CANISTER_ACTIVE, MFD_CANISTER_MISPLACED],
}
NULL_STRING = 'null'

# Drug Inventory Constants
EPBM_DOSEPACK_DEPARTMENT: str = "DOSEPACK"
EPBM_VIBRANT_DEPARTMENT: str = "VIBRANT"
DRUG_INVENTORY_MANAGEMENT_GROUP_ID: int = 38
DRUG_INVENTORY_MANAGEMENT_GROUP_NAME: str = "DrugInventoryManagement"
DRUG_INVENTORY_MANAGEMENT_ADDED_ID: int = 153
DRUG_INVENTORY_MANAGEMENT_ADDED_NAME: str = "Added"
DRUG_INVENTORY_MANAGEMENT_UPDATED_ID: int = 154
DRUG_INVENTORY_MANAGEMENT_UPDATED_NAME: str = "Updated"
DRUG_INVENTORY_MANAGEMENT_DELETED_ID: int = 155
DRUG_INVENTORY_MANAGEMENT_DELETED_NAME: str = "Deleted"
DRUG_INVENTORY_MANAGEMENT_ORDER_PENDING_ID: int = 196
DRUG_INVENTORY_MANAGEMENT_ORDER_PENDING_NAME: str = "Order Pending"
DRUG_INVENTORY_MANAGEMENT_ADDED_TO_CART_ID: int = 197
DRUG_INVENTORY_MANAGEMENT_ADDED_TO_CART_NAME: str = "Added to Cart"
DRUG_INVENTORY_MANAGEMENT_BYPASSED: int = 246
DRUG_INVENTORY_MANAGEMENT_BYPASSED_NAME: str = "Order Bypassed"
EPBM_INVENTORY_COUCH_DB: str = "epbm_inventory"
FILTER_PRE_ORDER_EPBM_BATCH_ORDER_DATA: str = "pre_order"
FILTER_AVAILABLE_EPBM_BATCH_ORDER_DATA: str = "available"
EPBM_TXR_LIST_LIMIT: int = 550
EPBM_NDC_LIST_LIMIT: int = 300
MFD_TROLLEY_DEVICE_TYPE = 8

# Local Drug Inventory Management Constants
LOCAL_DI_GROUP_ID: int = 44
LOCAL_DI_GROUP_NAME: str = "LocalDrugInventory"
LOCAL_DI_QTY_ADDED_ACTION_ID: int = 46
LOCAL_DI_QTY_ADDED_ACTION_NAME: str = "Quantity Added"
LOCAL_DI_QTY_CONSUMED_ACTION_ID: int = 47
LOCAL_DI_QTY_CONSUMED_ACTION_NAME: str = "Quantity Consumed"
LOCAL_DI_QTY_ADDED_FOR_SAME_FNDC_TXR_ACTION_ID: int = 48
LOCAL_DI_QTY_ADDED_FOR_SAME_FNDC_TXR_ACTION_NAME: str = "Quantity Added for same FNDC TXR"
LOCAL_DI_QTY_CONSUMED_FOR_SAME_FNDC_TXR_ACTION_ID: int = 49
LOCAL_DI_QTY_CONSUMED_FOR_SAME_FNDC_TXR_ACTION_NAME: str = "Quantity Consumed for same FNDC TXR"
LOCAL_DI_CONSUMED_FOR_PACK_ID_MSG: str = "Consumed for the pack_id: {}"
LOCAL_DI_CONSUMED_FOR_PACK_ID_AND_NDC_MSG: str = "Consumed for the pack_id: {} and NDC: {}"
LOCAL_DI_DROPPED_NDC_ADDED_WITH_ZERO_QTY_MSG: str = "For pack_id: {}, the dropped NDC: {} added with 0 qty in the " \
                                                    "local inventory."
LOCAL_DI_CONSUMED_MORE_THAN_AVAILABLE_MSG: str = "For pack_id: {}, the dropped NDC consumed more than the total " \
                                                 "available qty for the fndc_txr in the local inventory."
LOCAL_DI_QTY_ADDED_TO_NEGATIVE_FOR_SAME_NDC_MSG: str = "Added quantity to the negative inventory qty for same NDC."
LOCAL_DI_QTY_ADDED_TO_NEGATIVE_FOR_SAME_FNDC_TXR_MSG: str = "Added quantity of NDC {} to the negative inventory " \
                                                            "qty for same FNDC TXR."

# Static Value constants for Manual
MANUAL_USER_COUNT: str = "MANUAL_USER_COUNT"
MANUAL_PER_HOUR: str = "MANUAL_PER_HOUR"
MANUAL_PER_DAY_HOURS: str = "MANUAL_PER_DAY_HOURS"
MANUAL_SATURDAY_HOURS: str = "MANUAL_SATURDAY_HOURS"
MANUAL_SUNDAY_HOURS: str = "MANUAL_SUNDAY_HOURS"
MANUAL_CAPACITY: List[str] = [MANUAL_USER_COUNT, MANUAL_PER_HOUR, MANUAL_PER_DAY_HOURS, MANUAL_SATURDAY_HOURS,
                              MANUAL_SUNDAY_HOURS]

# static value constants for ordering
NUMBER_OF_DAYS_TO_ORDER: str = "NUMBER_OF_DAYS_TO_ORDER"
NUMBER_OF_DAYS_TO_FETCH_PO: str = "NUMBER_OF_DAYS_TO_FETCH_PO"
BUFFER_FOR_FAST_MOVING: str = "BUFFER_FOR_FAST_MOVING"
BUFFER_FOR_MEDIUM_FAST_MOVING: str = "BUFFER_FOR_MEDIUM_FAST_MOVING"
BUFFER_FOR_MEDIUM_SLOW_MOVING: str = "BUFFER_FOR_MEDIUM_SLOW_MOVING"
BUFFER_FOR_SLOW_MOVING: str = "BUFFER_FOR_SLOW_MOVING"
USAGE_TO_BUFFER: Dict[int, str] = {USAGE_FAST_MOVING: BUFFER_FOR_FAST_MOVING,
                                   USAGE_MEDIUM_FAST_MOVING: BUFFER_FOR_MEDIUM_FAST_MOVING,
                                   USAGE_MEDIUM_SLOW_MOVING: BUFFER_FOR_MEDIUM_SLOW_MOVING,
                                   USAGE_SLOW_MOVING: BUFFER_FOR_SLOW_MOVING}

# ExtPackStatus
EXT_PACK_STATUS_GROUP_ID = 41
EXT_PACK_STATUS_CODE_DELETED = 161
EXT_PACK_STATUS_CODE_CHANGE_RX_HOLD = 221
EXT_PACK_STATUS_CODE_DONE = 233
IPS_PACK_ERROR_CODE = 234
# process rx file codes
ERROR_CODE_CONNECTION_ISSUE_WHILE_FILE_SAVING = 5014
ERROR_CODE_CONNECTION_ISSUE_AFTER_FILE_SAVED = 5015
# Constants to set the default value for the reason on why a particular pack got deleted and communicate the same to IPS
# If Pack is deleted from DosePack portal then we will always have a delete-reason entered by the user, but we need to
# set the static delete reason for packs being deleted that have shared MFD
DELETE_PACK_DESC_SHARED_MFD = "This pack is deleted as it shares the MFD with the deleted Pack Display ID(s): {}."

LABEL_COUNT_JOB_DOCUMENT_CONSTANT = 'label_count'

# MVS Reason Constants
MVS_REASON_GROUP = 43

MVS_PRINTER_SLOW = 5
MVS_HEAT_SEAL_NOT_WORKING = 6
MVS_UNLOADING_STATION_NOT_WORKING = 7
MVS_DRUGS_OUT_OF_STOCK = 8
MVS_MULTIPLE_LABEL = 9
MVS_MISSING_PILLS = 10
MVS_OTHER = 11

# ExtChangeRx
EXTCHANGERXACTIONS_GROUP_ID = 42  # ExtChangeRx --> Actions Group(Reference in action_master)
EXTCHANGERXACTIONS_ADD = 44  # ChangeRx Request From IPS = Add
EXTCHANGERXACTIONS_DELETE = 45  # ChangeRx Request From IPS = Delete

EXTCHANGERX_PACK_DELETE_DESC = "Pack deletion due to processing of ChangeRx after Template processing is done."

# pack Pre-Processing sequence
PPP_SEQUENCE_IN_PROGRESS: int = 1
PPP_SEQUENCE_IN_PENDING: int = 0
PPP_SEQUENCE_DONE: int = 7
PPP_SEQUENCE_SAVE_ALTERNATES: int = 2
PPP_SEQUENCE_PACK_DISTRIBUTION_BY_BATCH_DONE: int = 3
PPP_SEQUENCE_MFD_RECOMMENDATION_DONE: int = 4
PPP_SEQUENCE_UPDATE_MFD_ANALYSIS_DONE: int = 5
PPP_SEQUENCE_CANISTER_RECOMMENDATION_DONE: int = 6

# pack pre-processing module mapping
PRE_PROCESSING_MODULE_MAPPING = {PPP_SEQUENCE_IN_PENDING: 0,
                                 PPP_SEQUENCE_PACK_DISTRIBUTION_BY_BATCH_DONE: 1,
                                 PPP_SEQUENCE_SAVE_ALTERNATES: 1,
                                 PPP_SEQUENCE_MFD_RECOMMENDATION_DONE: 2,
                                 PPP_SEQUENCE_UPDATE_MFD_ANALYSIS_DONE: 3,
                                 PPP_SEQUENCE_CANISTER_RECOMMENDATION_DONE: 4,
                                 PPP_SEQUENCE_DONE: 0}
PRE_PROCESSING_MODULE_BATCH_STATUS = [settings.BATCH_PENDING, settings.BATCH_CANISTER_TRANSFER_RECOMMENDED,
                                      settings.BATCH_ALTERNATE_DRUG_SAVED, settings.BATCH_MFD_USER_ASSIGNED,
                                      settings.BATCH_CANISTER_TRANSFER_DONE]

PDT_BATCH_GENERATION = 1
PDT_PRE_PROCESSING_ALTERNATE_DRUG_UPDATE = 2
PDT_PACK_FILL_WORKFLOW = 3
PDT_MVS_MANUAL_FILL = 4
PDT_REPLENISH_ALTERNATES = 5
PDT_MFD_ALTERNATE = 6
PDT_ALTERNATE_UNDEFINED = 7
PDT_CHANGE_RX_FLOW = 8
PDT_SLOT_TRANSACTION = 9
PDT_PRN_FILL_WORKFLOW = 10
PDT_FILL_ERROR_WORKFLOW = 11

SAME_NDC = 1
SAME_DRUG = 2
ALTERNATE_DRUG = 3


REQUIRED_CANISTER_COUNT = 4
MINIMUM_FILL_QUANTITY_REPLENISH_TEST = 30
# requested canister status
REQUESTED_CANISTER_STATUS_GROUP_ID: int = 45
APPROVED_ID: int = 199
IN_PROGRESS_ID: int = 200
DONE_ID: int = 201
CANCELLED_ID: int = 202
REJECTED_ID: int = 203
PENDING_ID: int = 204

# requested canister status list(to consider requested canister count for this status)
REQUESTED_CANISTER_STATUS_LIST = [APPROVED_ID, IN_PROGRESS_ID, PENDING_ID, DONE_ID]
# Recommended stick canister status
RECOMMENDED_STICK_CANISTER_STATUS_GROUP_ID: int = 46

CANISTER_DEFECTIVE_ID: int = 205
CANISTER_NOT_AVAILABLE_ID: int = 206

TEMP_MFD_LOGS: str = "TEMP_MFD_LOGS"

# Canister Tracker Usage Consideration
CANISTER_TRACKER_USAGE_CONSIDERATION_GROUP_ID = 48
USAGE_CONSIDERATION_PENDING = 207
USAGE_CONSIDERATION_IN_PROGRESS = 208
USAGE_CONSIDERATION_DONE = 209
USAGE_CONSIDERATION_DISCARD = 252

ERROR_CODE_INVALID_TOKEN = 5018
ERROR_CODE_INVALID_USER_TOKEN = 5019

# Constant to use for successful response to IPS
IPS_STATUS_CODE_OK = 200

ERROR_CODE_INVALID_USER_IPS_DOSEPACK = 5006
ERROR_CODE_NO_TEMPLATES_FOUND = 21000
ERROR_CODE_INTERNAL_CHANGE_RX = 21001
INVALID_API_CALL = 21002
ERROR_CODE_DEFAULT = 0

# Couch DB document to store information related to Change Rx.
CHANGE_RX_TEMPLATE_COUCH_DB: str = "template-change-rx"

# Common message to display to end user on MVS receiving Change Rx
NOTIFICATION_EXT_CHANGE_RX_GENERAL = "Prescription for this pack has been changed. Please take necessary actions."

DELETE_REASON_EXT_CHANGE_RX = 'Deleted from Pharmacy Software due to Change Rx workflow.'
HOLD_REASON_EXT_CHANGE_RX = 'Hold Pack(s) due to Change Rx workflow'

EXT_CHANGE_RX_PACKS_DESC = "Prescription Changed Packs"

# drug wo dimension for last how many months
TOTAL_MONTHS = 3

TEMPLATE_SPLIT_COUNT_THRESHOLD = 6
MAX_SLOT_COUNT_THRESHOLD = 10
MAX_TEMPLATE_SPLIT_COUNT_THRESHOLD = 10
TEMPLATE_SPLIT_COUNT_THRESHOLD_UNIT = 3


UPDATE_ALT_CANISTER = 'Update Alternate Canister'
UPDATE_ALT_DRUG_CANISTER = 'Update Alternate Drug Canister'
UPDATE_ALT_DRUG = 'Update Alternate Drug'
SKIP_FOR_BATCH = 'Skip for Batch'
SKIP_FOR_PACKS = 'Skip for Packs'
PACK_ASSIGNED = 'Pack assigned'
NDC_CHANGE = 'Ndc change'
PRINT_LABEL = 'Print label'
PACK_FILLLED_PARTIALLY = 'Pack filled partially'
SCAN_LABEL = 'Scan label'
VERIFICATION = 'Verification'
OUT_FOR_DELIVERY = 'Out for delivery'
DELIVERED = 'Delivered'
REVERT_BATCH_OR_PACKS = "Revert Batch or Packs"
REPLENISH_REVERTED_PACKS = "Replenish Reverted Packs"
SKIP_REASON_DRUG_NOT_ENOUGH_DRUG_QUANTITY = 'Not enough drug quantity'

ACTION_ID_MAP = {
    UPDATE_ALT_CANISTER: 3,
    UPDATE_ALT_DRUG_CANISTER: 4,
    UPDATE_ALT_DRUG: 5,
    SKIP_FOR_BATCH: 6,
    PACK_ASSIGNED: 7,
    NDC_CHANGE: 8,
    PRINT_LABEL: 9,
    PACK_FILLLED_PARTIALLY: 10,
    SCAN_LABEL: 11,
    VERIFICATION: 12,
    OUT_FOR_DELIVERY: 13,
    DELIVERED: 14,
    SKIP_FOR_PACKS: 50,
    REVERT_BATCH_OR_PACKS: 51,
    REPLENISH_REVERTED_PACKS: 53
}

BATCH_CHANGE = 11

REVERT_PACK_FLAG_0 = 0
REVERT_PACK_FLAG_1 = 1

# revert batch
PPP_schedule_batch = "schedule_batch"
BD_select_facilities = "select_facilities"

# revert batch to which screen
REVERT_TO_PACK_PRE_PROCESSING = "schedule_batch"
REVERT_TO_BATCH_DISTRIBUTION = "batch_distribution"

# revert batch call from which screen
CALL_FROM_PACK_PRE_PROCESSING = "pack_pre_processing"
CALL_FROM_PACK_QUEUE = "pack_queue"
CALL_FROM_MANUAL_FILL_FLOW = "manual_fill_flow"


REVERT_BATCH_INFO_MESSAGE = {
    'CANISTER_TRANSFER': 'Please try to revert the packs after completing the Canister Transfer Flow.',
    'GUIDED_TRANSFER': 'Please try to revert the packs after completing the Guided Replenish Flow.',
    'PENDING_PACKS': 'Please try to revert the packs that are in Pending state.',
    'MFD_FILLING_PENDING': 'Please try to revert the packs that have MFD Filling in Pending state.',
    'UNFILLED_MFD_CANISTER': 'You can revert the packs of the patients that have their MFD filling in a pending state or skipped state.',
    'OTHER_PATIENT_PACK': 'You can revert the packs of the other patient(s) which are in pending state.',
}

# Constant for template_settings
SLOT_VOLUME_THRESHOLD_MARK = 'SLOT_VOLUME_THRESHOLD_MARK'
SPLIT_STORE_SEPARATE = 'SPLIT_STORE_SEPARATE'

IS_MODIFIED_MAP = {'RED': 1,
                   'YELLOW': 2,
                   'GREEN': 0}

# max record in print job couch db document
MAX_RECORD_IN_PRINT_JOB_DOC = 50

FREQUENT_MFD_DRUGS_STATUS_GROUP_ID = 51
PENDING_MFD_DRUD_DIMENSION_FLOW = 215
SKIPPED_MFD_DRUG_DIMENSION_FLOW = 216
DONE_MFD_DRUG_DIMENSION_FLOW = 217

# Version of Robot/functionality used by DosePacker
GENERATE_PACK_VERSION = 2

# Ext Pack Usage Group Master and Code Master constants
EXT_PACK_USAGE_STATUS_GROUP_ID = 52
EXT_PACK_USAGE_STATUS_DESC = "ExtPackUsageStatus"

EXT_PACK_USAGE_STATUS_NA_ID = 218
EXT_PACK_USAGE_STATUS_NA_DESC = "Not Required"

EXT_PACK_USAGE_STATUS_PENDING_ID = 219
EXT_PACK_USAGE_STATUS_PENDING_DESC = "Pending"

EXT_PACK_USAGE_STATUS_PROGRESS_ID = 288
EXT_PACK_USAGE_STATUS_PROGRESS_DESC = "Pack Reuse In Progress"

EXT_PACK_USAGE_STATUS_DONE_ID = 220
EXT_PACK_USAGE_STATUS_DONE_DESC = "Done"

EXT_PACK_USAGE_STATUS_RTS_DONE_ID = 289
EXT_PACK_USAGE_STATUS_RTS_DONE_DESC = "Pack RTS Done"

EXT_PACK_USAGE_STATUS_PACK_RESEALED_ID = 290
EXT_PACK_USAGE_STATUS_PACK_RESEALED_DESC = "Pack Resealed"

EXT_PACK_USAGE_STATUS_PACK_DISCARDED_ID = 291
EXT_PACK_USAGE_STATUS_PACK_DISCARDED_DESC = "Pack Discarded"

# Ext Drug Usage Group Master and Code Master constants
REUSE_DRUG_STATUS_GROUP_ID = 64
REUSE_DRUG_STATUS_GROUP_DESC = "ReuseDrugStatus"

REUSE_DRUG_STATUS_DRUG_REUSE_PENDING_ID = 282
REUSE_DRUG_STATUS_DRUG_REUSE_PENDING_DESC = "Drug Reuse Pending"

REUSE_DRUG_STATUS_DRUG_REUSE_IN_PROGRESS_ID = 283
REUSE_DRUG_STATUS_DRUG_REUSE_IN_PROGRESS_DESC = "Drug Reuse In Progress"

REUSE_DRUG_STATUS_DRUG_REUSE_DONE_ID = 284
REUSE_DRUG_STATUS_DRUG_REUSE_DONE_DESC = "Drug Reuse Done"

REUSE_DRUG_STATUS_DRUG_RTS_DONE_ID = 285
REUSE_DRUG_STATUS_DRUG_RTS_DONE_DESC = "Drug RTS Done"

REUSE_DRUG_STATUS_DRUG_RESEALED_ID = 286
REUSE_DRUG_STATUS_DRUG_RESEALED_DESC = "Drug Resealed"

REUSE_DRUG_STATUS_DRUG_DISCARDED_ID = 287
REUSE_DRUG_STATUS_DRUG_DISCARDED_DESC = "Drug Discarded"

EXT_PACK_STATUS_CODE_DELETED_RETURN_IN_PHARMACY_ID = 292
EXT_PACK_STATUS_CODE_DELETED_RETURN_IN_PHARMACY_DESC = "Returned in Pharmacy"

# Pack Delivery Status Group Master and Code Master constants
PACK_DELIVERY_STATUS_GROUP_ID = 66
PACK_DELIVERY_STATUS_GROUP_DESC = "PackDeliveryStatus"

PACK_DELIVERY_STATUS_INSIDE_PHARMACY_ID = 295
PACK_DELIVERY_STATUS_INSIDE_PHARMACY_DESC = "Ext Pack Inside the Pharmacy"

PACK_DELIVERY_STATUS_RETURN_FROM_THE_DELIVERY_ID = 296
PACK_DELIVERY_STATUS_RETURN_FROM_THE_DELIVERY_DESC = "Ext Pack Return From The Delivery"

DELETE_PACK_DESC_FOR_OLD_PACKS_CHANGE_RX = "Change Rx: Old Packs are deleted as New Packs are obtained for processing."
DELETE_PACK_DESC_FOR_OLD_PACKS_MVS_CHANGE_RX = "Change Rx: Old Packs are deleted while continuing on MVS"

DRUG_ID_KEY = "drug_id"
CHECK_DAW_KEY = "check_daw"

# status for skipped_mapped drug
# used in remote_tech_slot_details table
SKIPPED_AND_MAPPED_RTS_DRUG_STATUS_ID = 53
SKIPPED_AND_FOREIGN_OBJECT_MAPPED = 222
SKIPPED_AND_BROKEN_PILL_MAPPED = 223
SKIPPED_AND_MAPPED_BUT_USER_NOT_SURE = 224
SKIPPED_AND_SURE_MAPPED = 225
SKIPPED_AND_MAPPED_BUT_REFERENCE_IMAGE_NOT_AVAILABLE = 226
SKIPPED_AND_DELETED = 255

# status of replenish canister: skip or not
REPLENISH_CANISTER_STATUS_GROUP_ID = 54
REPLENISH_CANISTER_TRANSFER_SKIPPED = 227
REPLENISH_CANISTER_TRANSFER_NOT_SKIPPED = 228
SKIPPED_DUE_TO_MANUAL_PACK = 229

REFRESH_BATCH_DISTRIBUTION = 2
REFRESH_PACK_QUEUE = 3

AUTO_REFRESH_BATCH_UUID_KEY = "batch_uuid"
AUTO_REFRESH_QUEUE_UUID_KEY = "queue_uuid"





PHARMACY_PACK_STATUS_GROUP_ID = 56

PHARMACY_PACK_STATUS_NOT_FILLED = 239
PHARMACY_PACK_STATUS_FILL_ERROR = 240
PHARMACY_PACK_STATUS_FIX_ERROR = 241
PHARMACY_PACK_STATUS_FILLED = 242
PHARMACY_PACK_STATUS_CHECKED = 243
PHARMACY_PACK_STATUS_NOT_DELIVERED = 244
PHARMACY_PACK_STATUS_RETURNED_IN_PHARMACY = 297


PHARMACY_PACK_STATUS_DICT = {
    "Not Filled (Fill Data Missing)": PHARMACY_PACK_STATUS_NOT_FILLED,
    "Fill Error": PHARMACY_PACK_STATUS_FILL_ERROR,
    "Fix Error": PHARMACY_PACK_STATUS_FIX_ERROR,
    "Filled (Not Checked)": PHARMACY_PACK_STATUS_FILLED,
    "Checked": PHARMACY_PACK_STATUS_CHECKED,
    "Not Delivered": PHARMACY_PACK_STATUS_NOT_DELIVERED,
    "Returned in Pharmacy": PHARMACY_PACK_STATUS_RETURNED_IN_PHARMACY
}

# MODULE IDS REGARDING PORTAL SCREEN
MODULE_DRUG_DISPENSING_TEMPLATES = 1
MODULE_BATCH_DISTRIBUTION = 2
MODULE_PACK_PRE_PROCESSING = 3
MODULE_CONTROL_PANEL = 4
MODULE_PACK_QUEUE = 5
MODULE_PACK_MASTER = 6
MODULE_CANISTER_MASTER = 7
MODULE_PRINTER_DASHBOARD = 8
MODULE_REPORTS = 9
MODULE_ASSIGN_MANUAL_FILL_USER = 10
MODULE_PRN_OTHER_RX = 38
MODULE_FILL_ERROR_FLOW = 39
MODULE_MANUAL_PACK_FILLING = 19

# Module Ids regarding robot
MODULE_CANISTER_TRANSFER = 29
MODULE_CANISTER_TO_BE_REPLENISHED = 30
MODULE_REPLENISH_SKIPPED_CANISTER = 31
MODULE_MISSING_PILL = 32
MODULE_MISSING_CANISTER = 33
MODULE_MALFUNCTION_CANISTER = 34
MODULE_MFD_FLOW = 35
MODULE_MFD_MALFUNCTION_CANISTER = 36
MODULE_MFD_MISSING_CANISTER = 37

# skipped canister constants
GROUP_MASTER_SKIPPED_CANISTER = 57
SKIPPED_FROM_PACK_PRE_PROCESSING = 247
SKIPPED_DURING_MISSING_CANISTER = 248
SKIPPED_DURING_MISSING_PILL = 249
SKIPPED_DURING_MALFUNCTION = 250
SKIPPED_DURING_TO_BE_REPLENISH_SCREEN = 251
# SKIP_FOR_PACKS_SELECTED = 252

SKIPPED_MODULE_ID_DICT = {
    MODULE_PACK_PRE_PROCESSING: SKIPPED_FROM_PACK_PRE_PROCESSING,
    MODULE_MISSING_CANISTER: SKIPPED_DURING_MISSING_CANISTER,
    MODULE_MISSING_PILL: SKIPPED_DURING_MISSING_PILL,
    MODULE_MALFUNCTION_CANISTER: SKIPPED_DURING_MALFUNCTION,
    MODULE_CANISTER_TO_BE_REPLENISHED: SKIPPED_DURING_TO_BE_REPLENISH_SCREEN
}

SKIPPED_MODULE_NAME = {
    "Pack Pre-Processing": SKIPPED_FROM_PACK_PRE_PROCESSING,
    "Missing canister pop-up": SKIPPED_DURING_MISSING_CANISTER,
    "Missing pill pop-up": SKIPPED_DURING_MISSING_PILL,
    "Malfunction pop-up": SKIPPED_DURING_MALFUNCTION,
    "To be replenish screen": SKIPPED_DURING_TO_BE_REPLENISH_SCREEN
}

EXPIRED_CANISTER = 0
EXPIRES_SOON_CANISTER = 1
NORMAL_EXPIRY_CANISTER = 2
COATING_TYPE = ['Sugar Coated', 'Enteric Coated', 'Compression Coated', 'Film Coated', 'Gelatin Coated', 'Non Coated', 'Others']

REQUIRED_MFD_CART_MESSAGE = "The packs in the current queue require {} Transfer to {}."

# API failure name in arguments for sending mail
DAILY_SHIPPED_CANISTERS_CHECK = "Shipped Canisters Check"
DAILY_DELIVERED_CANISTERS_CHECK = "Delivered Canisters Check"

CHEWABLE_TABLE =2
ORRALY_DISINTEGRATING_TABLET = 3
SOFT_GEL = 4

# delicate pill canister flow
DELICATE_DOSAGE_TYPES = [CHEWABLE_TABLE, ORRALY_DISINTEGRATING_TABLET, SOFT_GEL]
DELICATE_COATING_TYPES = ['Sugar Coated', 'Enteric Coated', 'Compression Coated', 'Film Coated', 'Gelatin Coated']
REGULAR_CANISTER = 0
DELICATE_CANISTER = 1
ROBOT_TO_ROBOT_TRANSFERS = 1
REGULAR_TRANSFERS = 0
MAX_DELICATE_DRAWER_LEVEL = 4
MIN_DELICATE_DRAWER_LEVEL = 2
LIMIT_FOR_TRANSFERS = 12

ROBOT_DRAWER_LEVELS = {"M": 1, "A": 2, "B": 3, "C": 4, "D": 5, "E": 6, "F": 7, "G": 8, "H": 9, "I": 10, "J": 11}

ROBOT_MAPPING = {2: "A",
                 3: "B"}

POWDER_PILL = 1
NOT_POWDER_PILL = 2
NOT_SURE = 3

# Module Ids
MODULE_PACK_QUEUE = 5

# PRN VIAL and OTHER STATUS
# Order status
ORDER_STATUS_GROUP_ID = 63
PENDING_ORDER = 279
SHIPPED_ORDER = 280
DELIVERED_ORDER = 281

PACKAGING_TYPE_GROUP_ID = 55
PACKAGING_TYPE_BLISTER_PACK_MULTI_7X4 = 237
PACKAGING_TYPE_BLISTER_PACK_UNIT_7X4 = 311
PACKAGING_TYPE_BLISTER_PACK_MULTI_8X4 = 312
PACKAGING_TYPE_BLISTER_PACK_UNIT_8X4 = 313
PACKAGING_TYPE_DRUG_BOTTLE = 238
PACKAGING_TYPE_VIAL = 274
PACKAGING_TYPE_BUBBLE_PACK = 270
PACKAGING_TYPE_OTHER = 271
PACKAGING_TYPES = {"Bubble": PACKAGING_TYPE_BUBBLE_PACK,
                   "Blister": PACKAGING_TYPE_BLISTER_PACK_MULTI_7X4,
                   "Vial": PACKAGING_TYPE_VIAL,
                   "Other": PACKAGING_TYPE_OTHER}
PACK_WISE_AVAILABLE_QUANTITY = {
    270: 31,
    237: 28
}
PACKAGING_TYPE_BLISTER_PACK_STRING = "Blister"
PACKAGING_TYPE_BUBBLE_PACK_STRING = "Bubble"
PRN_FILE_NAME_CONSTANT = "PRN#"
RETAIL_FILE_NAME_CONSTANT = "RETAIL#"

PRN_DONE_STATUS = 272
UNIT_PACK_TOTAL_QUANTITY = 28
ORDER_STATUS_LIST = [PENDING_ORDER, SHIPPED_ORDER, DELIVERED_ORDER]
NULL_ROUTE_NAME_ID = -100
NULL_ROUTE_NAME = "No Route Defined"

PACK_GRID_ROW_7x4 = 277
PACK_GRID_ROW_8x4 = 278
# Pack Grid
PACK_GRID_ROW_MAP = {
    PACK_GRID_ROW_7x4: [0, 1, 2, 3, 4, 5, 6],
    PACK_GRID_ROW_8x4: [0, 1, 2, 3, 4, 5, 6, 7]
}
PACKAGING_TYPES_CODE = {PACKAGING_TYPE_BUBBLE_PACK: "Bubble",
                        PACKAGING_TYPE_BLISTER_PACK_MULTI_7X4: "Blister",
                        PACKAGING_TYPE_VIAL: "Vial",
                        PACKAGING_TYPE_OTHER: "Other",
                        PACKAGING_TYPE_BLISTER_PACK_MULTI_8X4: "Blister",
                        PACKAGING_TYPE_BLISTER_PACK_UNIT_7X4: "Blister",
                        PACKAGING_TYPE_BLISTER_PACK_UNIT_8X4: "Blister"}

PRN_DUMMY_FILE = "PRN_DRUG_SYNC"

drug_schedule_list = {
    "0": 0,
    "2": 2,
    "3": 3,
    "4": 4,
    "5": 5
}

MIGRATION_FOLDER_PATH = "/home/meditab/PycharmProjects/new/dpws/migrations"
DUPLICATE_MIGRATION = "DUPLICATE_MIGRATION"

PILL_MISSING_REASON = "Pill Missing/Extra"
DOCTOR_APPROVAL_REASON = "No Doctor Approval"

FILL_ERROR_REASON = "Pack Error Reported from Pharmacy Software"
FIX_ERROR_REASON = "Pack Error Resolved in Dosepack"

GROUP_MASTER_STORE_TYPE = 67

STORE_TYPE_CYCLIC = 300
STORE_TYPE_NON_CYCLIC = 301
STORE_TYPE_RETAIL = 302

RPH_VERIFICATION_STATUS_NOT_CHECKED = 303
RPH_VERIFICATION_STATUS_CHECKED = 304
RPH_VERIFICATION_STATUS_ON_HOLD_SYSTEM = 305
RPH_VERIFICATION_STATUS_ON_HOLD_MANUAL = 306
RPH_VERIFICATION_STATUS_FILL_ERROR = 293
RPH_VERIFICATION_STATUS_FIX_ERROR = 294

GROUP_MASTER_RPH_VERIFICATION_STATUS = 68

GROUP_MASTER_DELIVERY_STATUS = 69

DELIVERY_STATUS_NOT_DELIVERED = 307
DELIVERY_STATUS_TO_BE_DELIVERED = 308
DELIVERY_STATUS_DELIVERED = 309
DELIVERY_STATUS_CANCELLED = 310

DELIVERY_STATUS_MAP = {1: 307,
                       2: 308,
                       3: 309,
                       4: 310,
                       5: 307}

FILL_CYCLE_WEEKLY = 46

QUEUE_TYPE_MAP = {"W": "Waiting",
                  "P": "Pickup",
                  "D": "Delivery",
                  "M": "Mail/Ship"}
