"""
this migration will take prs_drug_details table data from rts-prs database, currently it is dpws_dev_07,and update status in production database.
"""
import json

from playhouse.shortcuts import case

from dosepack.base_model.base_model import db
from model.model_init import init_db
from src import constants
from src.model.model_prs_drug_details import PRSDrugDetails


def store_prs_drug_details_data():
    try:
        unique_drug_staus_dict = dict()

        query = PRSDrugDetails.select(PRSDrugDetails.unique_drug_id,
                                      PRSDrugDetails.status).dicts() \
            .where(PRSDrugDetails.status != constants.PRS_DRUG_STATUS_PENDING)

        for data in query:
            unique_drug_staus_dict[data["unique_drug_id"]] = (data["unique_drug_id"], data["status"])

        print(f'unique_drug_staus_dict: {unique_drug_staus_dict}')
        print(f'len of unique_drug_staus_dict: {len(unique_drug_staus_dict)}')

        return unique_drug_staus_dict
    except Exception as e:
        print(f'In store_prs_drug_details_data, e: {e}')


def update_prs_drug_details_status():
    try:
        update_unique_drug_staus_dict = {87: (87, 91), 5641: (5641, 91), 5772: (5772, 91), 6139: (6139, 91),
                                         6158: (6158, 91), 6161: (6161, 91), 6177: (6177, 91), 6183: (6183, 91),
                                         987: (987, 92), 1598: (1598, 92), 2761: (2761, 92), 3814: (3814, 92),
                                         5458: (5458, 92), 5459: (5459, 92), 5694: (5694, 92), 5924: (5924, 92),
                                         5923: (5923, 93), 33: (33, 94), 34: (34, 94), 64: (64, 94), 69: (69, 94),
                                         74: (74, 94), 163: (163, 94), 165: (165, 94), 236: (236, 94), 238: (238, 94),
                                         312: (312, 94), 358: (358, 94), 391: (391, 94), 433: (433, 94), 434: (434, 94),
                                         437: (437, 94), 438: (438, 94), 439: (439, 94), 460: (460, 94), 461: (461, 94),
                                         474: (474, 94), 498: (498, 94), 504: (504, 94), 505: (505, 94), 514: (514, 94),
                                         517: (517, 94), 557: (557, 94), 558: (558, 94), 568: (568, 94), 571: (571, 94),
                                         594: (594, 94), 595: (595, 94), 597: (597, 94), 628: (628, 94), 629: (629, 94),
                                         638: (638, 94), 739: (739, 94), 740: (740, 94), 743: (743, 94), 744: (744, 94),
                                         753: (753, 94), 754: (754, 94), 821: (821, 94), 822: (822, 94), 826: (826, 94),
                                         831: (831, 94), 843: (843, 94), 870: (870, 94), 879: (879, 94), 880: (880, 94),
                                         881: (881, 94), 882: (882, 94), 883: (883, 94), 886: (886, 94), 887: (887, 94),
                                         909: (909, 94), 912: (912, 94), 918: (918, 94), 951: (951, 94), 952: (952, 94),
                                         968: (968, 94), 969: (969, 94), 972: (972, 94), 973: (973, 94), 980: (980, 94),
                                         985: (985, 94), 990: (990, 94), 991: (991, 94), 1002: (1002, 94),
                                         1013: (1013, 94), 1015: (1015, 94), 1028: (1028, 94), 1031: (1031, 94),
                                         1035: (1035, 94), 1036: (1036, 94), 1037: (1037, 94), 1053: (1053, 94),
                                         1072: (1072, 94), 1080: (1080, 94), 1093: (1093, 94), 1094: (1094, 94),
                                         1095: (1095, 94), 1097: (1097, 94), 1102: (1102, 94), 1150: (1150, 94),
                                         1178: (1178, 94), 1203: (1203, 94), 1204: (1204, 94), 1252: (1252, 94),
                                         1258: (1258, 94), 1267: (1267, 94), 1273: (1273, 94), 1277: (1277, 94),
                                         1278: (1278, 94), 1279: (1279, 94), 1291: (1291, 94), 1295: (1295, 94),
                                         1320: (1320, 94), 1350: (1350, 94), 1357: (1357, 94), 1382: (1382, 94),
                                         1426: (1426, 94), 1427: (1427, 94), 1428: (1428, 94), 1430: (1430, 94),
                                         1432: (1432, 94), 1453: (1453, 94), 1454: (1454, 94), 1455: (1455, 94),
                                         1461: (1461, 94), 1517: (1517, 94), 1518: (1518, 94), 1546: (1546, 94),
                                         1548: (1548, 94), 1549: (1549, 94), 1555: (1555, 94), 1579: (1579, 94),
                                         1597: (1597, 94), 1599: (1599, 94), 1600: (1600, 94), 1601: (1601, 94),
                                         1605: (1605, 94), 1607: (1607, 94), 1608: (1608, 94), 1609: (1609, 94),
                                         1696: (1696, 94), 1763: (1763, 94), 1765: (1765, 94), 1773: (1773, 94),
                                         1778: (1778, 94), 1780: (1780, 94), 1781: (1781, 94), 1794: (1794, 94),
                                         1831: (1831, 94), 1832: (1832, 94), 1838: (1838, 94), 1840: (1840, 94),
                                         1842: (1842, 94), 1849: (1849, 94), 1863: (1863, 94), 1875: (1875, 94),
                                         1879: (1879, 94), 1881: (1881, 94), 1883: (1883, 94), 1886: (1886, 94),
                                         1893: (1893, 94), 1894: (1894, 94), 1904: (1904, 94), 2007: (2007, 94),
                                         2006: (2006, 94), 2043: (2043, 94), 2044: (2044, 94), 2052: (2052, 94),
                                         2094: (2094, 94), 2123: (2123, 94), 2126: (2126, 94), 2127: (2127, 94),
                                         2134: (2134, 94), 2135: (2135, 94), 2140: (2140, 94), 2141: (2141, 94),
                                         2152: (2152, 94), 2156: (2156, 94), 2204: (2204, 94), 2205: (2205, 94),
                                         2206: (2206, 94), 2207: (2207, 94), 2208: (2208, 94), 2209: (2209, 94),
                                         2210: (2210, 94), 2217: (2217, 94), 2220: (2220, 94), 2222: (2222, 94),
                                         2223: (2223, 94), 2224: (2224, 94), 2227: (2227, 94), 2229: (2229, 94),
                                         2230: (2230, 94), 2231: (2231, 94), 2232: (2232, 94), 2235: (2235, 94),
                                         2236: (2236, 94), 2237: (2237, 94), 2238: (2238, 94), 2239: (2239, 94),
                                         2240: (2240, 94), 2241: (2241, 94), 2243: (2243, 94), 2244: (2244, 94),
                                         2246: (2246, 94), 2275: (2275, 94), 2276: (2276, 94), 2285: (2285, 94),
                                         2293: (2293, 94), 2294: (2294, 94), 2295: (2295, 94), 2296: (2296, 94),
                                         2297: (2297, 94), 2301: (2301, 94), 2303: (2303, 94), 2304: (2304, 94),
                                         2320: (2320, 94), 2321: (2321, 94), 2322: (2322, 94), 2325: (2325, 94),
                                         2326: (2326, 94), 2329: (2329, 94), 2403: (2403, 94), 2404: (2404, 94),
                                         2408: (2408, 94), 2414: (2414, 94), 2415: (2415, 94), 2416: (2416, 94),
                                         2424: (2424, 94), 2426: (2426, 94), 2427: (2427, 94), 2430: (2430, 94),
                                         2442: (2442, 94), 2444: (2444, 94), 2450: (2450, 94), 2459: (2459, 94),
                                         2461: (2461, 94), 2462: (2462, 94), 2463: (2463, 94), 2464: (2464, 94),
                                         2467: (2467, 94), 2468: (2468, 94), 2476: (2476, 94), 2477: (2477, 94),
                                         2494: (2494, 94), 2499: (2499, 94), 2500: (2500, 94), 2505: (2505, 94),
                                         2506: (2506, 94), 2508: (2508, 94), 2589: (2589, 94), 2594: (2594, 94),
                                         2595: (2595, 94), 2639: (2639, 94), 2642: (2642, 94), 2643: (2643, 94),
                                         2648: (2648, 94), 2650: (2650, 94), 2677: (2677, 94), 2687: (2687, 94),
                                         2698: (2698, 94), 2701: (2701, 94), 2720: (2720, 94), 2721: (2721, 94),
                                         2722: (2722, 94), 2724: (2724, 94), 2726: (2726, 94), 2728: (2728, 94),
                                         2729: (2729, 94), 2730: (2730, 94), 2731: (2731, 94), 2733: (2733, 94),
                                         2737: (2737, 94), 2738: (2738, 94), 2739: (2739, 94), 2742: (2742, 94),
                                         2743: (2743, 94), 2744: (2744, 94), 2830: (2830, 94), 2839: (2839, 94),
                                         2853: (2853, 94), 2854: (2854, 94), 2855: (2855, 94), 2856: (2856, 94),
                                         2869: (2869, 94), 2890: (2890, 94), 2961: (2961, 94), 2966: (2966, 94),
                                         2967: (2967, 94), 2971: (2971, 94), 2975: (2975, 94), 2979: (2979, 94),
                                         2980: (2980, 94), 3035: (3035, 94), 3041: (3041, 94), 3042: (3042, 94),
                                         3082: (3082, 94), 3093: (3093, 94), 3094: (3094, 94), 3095: (3095, 94),
                                         3102: (3102, 94), 3103: (3103, 94), 3104: (3104, 94), 3114: (3114, 94),
                                         3143: (3143, 94), 3146: (3146, 94), 3148: (3148, 94), 3216: (3216, 94),
                                         3217: (3217, 94), 3219: (3219, 94), 3228: (3228, 94), 3229: (3229, 94),
                                         3230: (3230, 94), 3232: (3232, 94), 3243: (3243, 94), 3247: (3247, 94),
                                         3267: (3267, 94), 3268: (3268, 94), 3269: (3269, 94), 3275: (3275, 94),
                                         3325: (3325, 94), 3327: (3327, 94), 3330: (3330, 94), 3350: (3350, 94),
                                         3363: (3363, 94), 3384: (3384, 94), 3397: (3397, 94), 3415: (3415, 94),
                                         3423: (3423, 94), 3457: (3457, 94), 3458: (3458, 94), 3459: (3459, 94),
                                         3460: (3460, 94), 3464: (3464, 94), 3465: (3465, 94), 3466: (3466, 94),
                                         3476: (3476, 94), 3477: (3477, 94), 3486: (3486, 94), 3488: (3488, 94),
                                         3532: (3532, 94), 3533: (3533, 94), 3537: (3537, 94), 3538: (3538, 94),
                                         3549: (3549, 94), 3550: (3550, 94), 3551: (3551, 94), 3552: (3552, 94),
                                         3555: (3555, 94), 3566: (3566, 94), 3569: (3569, 94), 3578: (3578, 94),
                                         3598: (3598, 94), 3621: (3621, 94), 3622: (3622, 94), 3654: (3654, 94),
                                         3656: (3656, 94), 3684: (3684, 94), 3690: (3690, 94), 3715: (3715, 94),
                                         3716: (3716, 94), 3722: (3722, 94), 3772: (3772, 94), 3773: (3773, 94),
                                         3778: (3778, 94), 3782: (3782, 94), 3806: (3806, 94), 3808: (3808, 94),
                                         3811: (3811, 94), 3813: (3813, 94), 3817: (3817, 94), 3819: (3819, 94),
                                         3821: (3821, 94), 3823: (3823, 94), 3847: (3847, 94), 3852: (3852, 94),
                                         3853: (3853, 94), 3854: (3854, 94), 3855: (3855, 94), 3856: (3856, 94),
                                         3857: (3857, 94), 3860: (3860, 94), 3861: (3861, 94), 3879: (3879, 94),
                                         3880: (3880, 94), 3881: (3881, 94), 3882: (3882, 94), 3933: (3933, 94),
                                         3949: (3949, 94), 3950: (3950, 94), 3951: (3951, 94), 3952: (3952, 94),
                                         3974: (3974, 94), 3975: (3975, 94), 3978: (3978, 94), 3983: (3983, 94),
                                         3993: (3993, 94), 3994: (3994, 94), 3995: (3995, 94), 3997: (3997, 94),
                                         4003: (4003, 94), 4006: (4006, 94), 4007: (4007, 94), 4009: (4009, 94),
                                         4010: (4010, 94), 4012: (4012, 94), 4016: (4016, 94), 4017: (4017, 94),
                                         4018: (4018, 94), 4025: (4025, 94), 4038: (4038, 94), 4059: (4059, 94),
                                         4060: (4060, 94), 4061: (4061, 94), 4062: (4062, 94), 4063: (4063, 94),
                                         4109: (4109, 94), 4110: (4110, 94), 4117: (4117, 94), 4118: (4118, 94),
                                         4130: (4130, 94), 4131: (4131, 94), 4132: (4132, 94), 4157: (4157, 94),
                                         4160: (4160, 94), 4164: (4164, 94), 4171: (4171, 94), 4176: (4176, 94),
                                         4209: (4209, 94), 4210: (4210, 94), 4211: (4211, 94), 4224: (4224, 94),
                                         4225: (4225, 94), 4226: (4226, 94), 4227: (4227, 94), 4228: (4228, 94),
                                         4229: (4229, 94), 4249: (4249, 94), 4266: (4266, 94), 4267: (4267, 94),
                                         4286: (4286, 94), 4287: (4287, 94), 4288: (4288, 94), 4289: (4289, 94),
                                         4290: (4290, 94), 4291: (4291, 94), 4307: (4307, 94), 4308: (4308, 94),
                                         4309: (4309, 94), 4310: (4310, 94), 4312: (4312, 94), 4316: (4316, 94),
                                         4321: (4321, 94), 4323: (4323, 94), 4335: (4335, 94), 4336: (4336, 94),
                                         4423: (4423, 94), 4424: (4424, 94), 4425: (4425, 94), 4426: (4426, 94),
                                         4427: (4427, 94), 4428: (4428, 94), 4429: (4429, 94), 4430: (4430, 94),
                                         4436: (4436, 94), 4438: (4438, 94), 4443: (4443, 94), 4470: (4470, 94),
                                         4471: (4471, 94), 4472: (4472, 94), 4487: (4487, 94), 4491: (4491, 94),
                                         4496: (4496, 94), 4506: (4506, 94), 4507: (4507, 94), 4508: (4508, 94),
                                         4509: (4509, 94), 4511: (4511, 94), 4512: (4512, 94), 4516: (4516, 94),
                                         4517: (4517, 94), 4518: (4518, 94), 4528: (4528, 94), 4529: (4529, 94),
                                         4546: (4546, 94), 4552: (4552, 94), 4553: (4553, 94), 4554: (4554, 94),
                                         4555: (4555, 94), 4556: (4556, 94), 4557: (4557, 94), 4558: (4558, 94),
                                         4560: (4560, 94), 4561: (4561, 94), 4562: (4562, 94), 4563: (4563, 94),
                                         4569: (4569, 94), 4570: (4570, 94), 4576: (4576, 94), 4577: (4577, 94),
                                         4579: (4579, 94), 4580: (4580, 94), 4581: (4581, 94), 4582: (4582, 94),
                                         4585: (4585, 94), 4586: (4586, 94), 4587: (4587, 94), 4588: (4588, 94),
                                         4589: (4589, 94), 4590: (4590, 94), 4591: (4591, 94), 4593: (4593, 94),
                                         4594: (4594, 94), 4595: (4595, 94), 4598: (4598, 94), 4599: (4599, 94),
                                         4618: (4618, 94), 4622: (4622, 94), 4623: (4623, 94), 4624: (4624, 94),
                                         4628: (4628, 94), 4631: (4631, 94), 4632: (4632, 94), 4652: (4652, 94),
                                         4662: (4662, 94), 4664: (4664, 94), 4709: (4709, 94), 4720: (4720, 94),
                                         4721: (4721, 94), 4723: (4723, 94), 4725: (4725, 94), 4726: (4726, 94),
                                         4729: (4729, 94), 4731: (4731, 94), 4737: (4737, 94), 4741: (4741, 94),
                                         4744: (4744, 94), 4751: (4751, 94), 4756: (4756, 94), 4757: (4757, 94),
                                         4759: (4759, 94), 4761: (4761, 94), 4772: (4772, 94), 4774: (4774, 94),
                                         4775: (4775, 94), 4776: (4776, 94), 4777: (4777, 94), 4778: (4778, 94),
                                         4779: (4779, 94), 4780: (4780, 94), 4784: (4784, 94), 4785: (4785, 94),
                                         4786: (4786, 94), 4792: (4792, 94), 4806: (4806, 94), 4833: (4833, 94),
                                         4866: (4866, 94), 4867: (4867, 94), 4881: (4881, 94), 4882: (4882, 94),
                                         4883: (4883, 94), 4885: (4885, 94), 4899: (4899, 94), 4900: (4900, 94),
                                         4902: (4902, 94), 4903: (4903, 94), 4989: (4989, 94), 4998: (4998, 94),
                                         4999: (4999, 94), 5000: (5000, 94), 5002: (5002, 94), 5004: (5004, 94),
                                         5005: (5005, 94), 5077: (5077, 94), 5095: (5095, 94), 5082: (5082, 94),
                                         5122: (5122, 94), 5135: (5135, 94), 5192: (5192, 94), 5123: (5123, 94),
                                         5106: (5106, 94), 5101: (5101, 94), 5229: (5229, 94), 5236: (5236, 94),
                                         5091: (5091, 94), 5114: (5114, 94), 5094: (5094, 94), 5092: (5092, 94),
                                         5090: (5090, 94), 5322: (5322, 94), 5119: (5119, 94), 5121: (5121, 94),
                                         5356: (5356, 94), 5089: (5089, 94), 5100: (5100, 94), 5109: (5109, 94),
                                         5441: (5441, 94), 5443: (5443, 94), 5450: (5450, 94), 5451: (5451, 94),
                                         5454: (5454, 94), 5456: (5456, 94), 5457: (5457, 94), 5463: (5463, 94),
                                         5467: (5467, 94), 5468: (5468, 94), 5473: (5473, 94), 5480: (5480, 94),
                                         5481: (5481, 94), 5484: (5484, 94), 5486: (5486, 94), 5491: (5491, 94),
                                         5495: (5495, 94), 5496: (5496, 94), 5500: (5500, 94), 5505: (5505, 94),
                                         5506: (5506, 94), 5507: (5507, 94), 5508: (5508, 94), 5509: (5509, 94),
                                         5513: (5513, 94), 5535: (5535, 94), 5552: (5552, 94), 5562: (5562, 94),
                                         5567: (5567, 94), 5571: (5571, 94), 5582: (5582, 94), 5586: (5586, 94),
                                         5593: (5593, 94), 5598: (5598, 94), 5603: (5603, 94), 5604: (5604, 94),
                                         5607: (5607, 94), 5608: (5608, 94), 5612: (5612, 94), 5614: (5614, 94),
                                         5616: (5616, 94), 5617: (5617, 94), 5620: (5620, 94), 5621: (5621, 94),
                                         5623: (5623, 94), 5626: (5626, 94), 5630: (5630, 94), 5633: (5633, 94),
                                         5634: (5634, 94), 5642: (5642, 94), 5645: (5645, 94), 5646: (5646, 94),
                                         5650: (5650, 94), 5656: (5656, 94), 5657: (5657, 94), 5662: (5662, 94),
                                         5675: (5675, 94), 5676: (5676, 94), 5703: (5703, 94), 5708: (5708, 94),
                                         5712: (5712, 94), 5713: (5713, 94), 5714: (5714, 94), 5715: (5715, 94),
                                         5716: (5716, 94), 5721: (5721, 94), 5724: (5724, 94), 5753: (5753, 94),
                                         5760: (5760, 94), 5788: (5788, 94), 5805: (5805, 94), 5832: (5832, 94),
                                         5837: (5837, 94), 5862: (5862, 94), 5872: (5872, 94), 5878: (5878, 94),
                                         5933: (5933, 94)}

        unique_drug_status_case = case(PRSDrugDetails.unique_drug_id, list(update_unique_drug_staus_dict.values()))

        status = PRSDrugDetails.update(status=unique_drug_status_case).where(
            PRSDrugDetails.unique_drug_id.in_(list(update_unique_drug_staus_dict.keys()))).execute()

        print(f"In update_prs_drug_details_status, status: {status}")
    except Exception as e:
        print(f'In update_prs_drug_details_status, e: {e}')


if __name__ == "__main__":
    init_db(db, "database_migration")
    # response = store_prs_drug_details_data()
    # print(response)

    update_prs_drug_details_status()
